@@ -1,4 +1,4 @@
-#ident "$Id: mount.c,v 1.4 1999/12/17 19:02:47 hpa Exp $"
+#ident "$Id: mount.c,v 1.2 2003/09/10 14:27:41 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  mount.c - Abstract mount code used by modules for an unexpected
@@ -20,13 +20,14 @@
  * ----------------------------------------------------------------------- */
 
 #include <syslog.h>
+#include <string.h>
 #include <stdlib.h>
 #include "automount.h"
 
 /* These filesystems are known not to work with the "generic" module */
 /* Note: starting with Samba 2.0.6, smbfs is handled generically.    */
 static char *not_generic[] = { "nfs", "ncpfs", "userfs", "afs",
-			       "autofs", "changer", NULL };
+			       "autofs", "changer", "bind", NULL };
 
 int do_mount(const char *root, const char *name, int name_len,
 	     const char *what, const char *fstype, const char *options)
