@@ -1,4 +1,4 @@
-#ident "$Id: spawn.c,v 1.2 1997/10/06 21:44:07 hpa Exp $"
+#ident "$Id: spawn.c,v 1.1.1.1 2003/09/09 11:21:57 raven Exp $"
 /* ----------------------------------------------------------------------- *
  * 
  *  spawn.c - run programs synchronously with output redirected to syslog
@@ -26,8 +26,8 @@
 
 #include "automount.h"
 
-/* Used by subprocesses to avoid carrying over the main daemon's rather
-   weird signalling environment */
+/* Used by subprocesses which exec to avoid carrying over the main
+   daemon's rather weird signalling environment */
 void reset_signals(void)
 {
   struct sigaction sa;
@@ -37,15 +37,60 @@ void reset_signals(void)
   sigfillset(&allsignals);
   sigprocmask(SIG_BLOCK, &allsignals, NULL);
   
-  sa.sa_handler = SIG_DFL;
+  /* Discard all pending signals */
+  sa.sa_handler = SIG_IGN;
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
   for ( i = 1 ; i < NSIG ; i++ )
     sigaction(i, &sa, NULL);
   
+  sa.sa_handler = SIG_DFL;
+  for ( i = 1 ; i < NSIG ; i++ )
+    sigaction(i, &sa, NULL);
+
   sigprocmask(SIG_UNBLOCK, &allsignals, NULL);
 }
 
+/* Used by subprocesses which don't exec to avoid carrying over the
+   main daemon's rather weird signalling environment.  Signals are
+   mostly ignored so that "/bin/kill -x automount" (where x is
+   typically SIGTERM or SIGUSR1) only affects the main process. */
+void ignore_signals(void)
+{
+  struct sigaction sa;
+  sigset_t allsignals;
+  int i;
+  
+  sigfillset(&allsignals);
+  sigprocmask(SIG_BLOCK, &allsignals, NULL);
+  
+  /* Discard all pending signals and ignore them */
+  sa.sa_handler = SIG_IGN;
+  sigemptyset(&sa.sa_mask);
+  sa.sa_flags = 0;
+  for ( i = 1 ; i < NSIG ; i++ )
+    sigaction(i, &sa, NULL);
+
+  /* Default handler for SIGCHLD so that waitpid() still works */
+  sa.sa_handler = SIG_DFL;
+  sigaction(SIGCHLD, &sa, NULL);
+
+  sigprocmask(SIG_UNBLOCK, &allsignals, NULL);
+}
+
+/* Throw away an unwanted pending signal */
+void discard_pending(int sig)
+{
+	struct sigaction sa, oldsa;
+
+	sa.sa_handler = SIG_IGN;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
+
+	sigaction(sig, &sa, &oldsa);
+	sigaction(sig, &oldsa, NULL);
+}
+
 #define ERRBUFSIZ 2047		/* Max length of error string excl \0 */
 
 int spawnv(int logpri, const char *prog, const char * const *argv) {
@@ -86,6 +131,7 @@ int spawnv(int logpri, const char *prog, const char * const *argv) {
 
     if ( f < 0 ) {
       close(pipefd[0]);
+      sigprocmask(SIG_SETMASK, &oldsig, NULL);
       return -1;
     }
 
