@@ -1,8 +1,18 @@
-/*
- * lookup_file.c
+#ident "$Id: lookup_file.c,v 1.1.1.1 2003/09/09 11:22:05 raven Exp $"
+/* ----------------------------------------------------------------------- *
+ *   
+ *  lookup_file.c - module for Linux automount to query a flat file map
  *
- * Module for Linux automountd to access a plain file automount map
- */
+ *   Copyright 1997 Transmeta Corporation - All Rights Reserved
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA 02139,
+ *   USA; either version 2 of the License, or (at your option) any later
+ *   version; incorporated herein by reference.
+ *
+ * ----------------------------------------------------------------------- */
+
 
 #include <stdio.h>
 #include <malloc.h>
@@ -24,13 +34,14 @@
 #define MAPENT_MAX_LEN 4095
 
 struct lookup_context {
-  char *mapname;
+  const char *mapname;
   struct parse_mod *parse;
 };
 
 int lookup_version = AUTOFS_LOOKUP_VERSION; /* Required by protocol */
 
-int lookup_init(char *mapfmt, int argc, char **argv, void **context)
+int lookup_init(const char *mapfmt, int argc, const char * const *argv,
+		void **context)
 {
   struct lookup_context *ctxt;
 
@@ -61,11 +72,13 @@ int lookup_init(char *mapfmt, int argc, char **argv, void **context)
   return !(ctxt->parse = open_parse(mapfmt,MODPREFIX,argc-1,argv+1));
 }
 
-int lookup_mount(char *root, char *name, int name_len, void *context)
+int lookup_mount(const char *root, const char *name, int name_len,
+		 void *context)
 {
   struct lookup_context *ctxt = (struct lookup_context *) context;
   int ch, nch;
-  char mapent[MAPENT_MAX_LEN+1], *p, *nptr;
+  char mapent[MAPENT_MAX_LEN+1], *p;
+  const char *nptr;
   int mapent_len;
   FILE *f;
   enum {
@@ -88,7 +101,7 @@ int lookup_mount(char *root, char *name, int name_len, void *context)
   gotten = got_nothing;
 
   /* Shut up gcc */
-  p = nptr = NULL;
+  nptr = p = NULL;
   mapent_len = 0;
   getting = got_nothing;
   escape = esc_none;
