@@ -1,10 +1,11 @@
-#ident "$Id: lookup_program.c,v 1.2 1997/10/06 21:52:02 hpa Exp $"
+#ident "$Id: lookup_program.c,v 1.1.1.1 2003/09/09 11:22:05 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  lookup_program.c - module for Linux automount to access an
  *                     automount map via a query program 
  *
  *   Copyright 1997 Transmeta Corporation - All Rights Reserved
+ *   Copyright 1999-2000 Jeremy Fitzhardinge <jeremy@goop.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -14,18 +15,14 @@
  *
  * ----------------------------------------------------------------------- */
 
-/*
-
- */
-
 #include <ctype.h>
 #include <errno.h>
-#include <limits.h>
 #include <malloc.h>
 #include <stdio.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <signal.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/times.h>
@@ -86,12 +83,16 @@ int lookup_mount(const char *root, const char *name, int name_len,
   struct lookup_context *ctxt = (struct lookup_context *) context;
   char mapent[MAPENT_MAX_LEN+1], *mapp;
   char errbuf[1024], *errp;
-  char *p, ch;
+  char ch;
   int pipefd[2], epipefd[2];
   pid_t f;
   int files_left;
   int status;
   fd_set readfds, ourfds;
+  enum state { st_space, st_map, st_done } state;
+  int quoted = 0;
+  int ret;
+  int max_fd;
 
   syslog(LOG_DEBUG, MODPREFIX "looking up %s", name);
 
@@ -117,6 +118,7 @@ int lookup_mount(const char *root, const char *name, int name_len,
     syslog(LOG_ERR, MODPREFIX "fork: %m");
     return 1;
   } else if ( f == 0 ) {
+    reset_signals();
     close(pipefd[0]);
     close(epipefd[0]);
     dup2(pipefd[1],STDOUT_FILENO);
@@ -131,31 +133,67 @@ int lookup_mount(const char *root, const char *name, int name_len,
 
   mapp = mapent;
   errp = errbuf;
+  state = st_space;
 
   FD_ZERO(&ourfds);
   FD_SET(pipefd[0],&ourfds);
   FD_SET(epipefd[0],&ourfds);
 
+  max_fd = pipefd[0] > epipefd[0] ? pipefd[0] : epipefd[0];
+
   files_left = 2;
 
-  while (files_left) {
+  while (files_left != 0) {
     readfds = ourfds;
-    if ( select(OPEN_MAX, &readfds, NULL, NULL, NULL) < 0 && 
+    if ( select(max_fd+1, &readfds, NULL, NULL, NULL) < 0 && 
 	 errno != EINTR )
       break;
-    
+
+    /* Parse maps from stdout */
     if ( FD_ISSET(pipefd[0],&readfds) ) {
       if ( read(pipefd[0], &ch, 1) < 1 ) {
 	FD_CLR(pipefd[0], &ourfds);
 	files_left--;
-      } else if ( mapp ) {
-	if ( ch == '\n' ) {
+      }
+      
+      if (!quoted && ch == '\\') {
+        quoted = 1;
+        continue;
+      }
+
+      switch(state)
+      {
+      case st_space:
+	if (quoted || !isspace(ch)) {
+	  *mapp++ = ch;
+	  state = st_map;
+	}
+	break;
+      case st_map:
+	if (!quoted && ch == '\n') {
 	  *mapp = '\0';
-	  mapp = NULL;		/* End of line reached */
-	} else if ( mapp-mapent < MAPENT_MAX_LEN )
-	  *(mapp++) = ch;
+	  state = st_done;
+	} else if (mapp - mapent < MAPENT_MAX_LEN-1) {
+	  /* Eat \ quoting \n, otherwise pass it through for the parser */
+	  if (quoted) {
+	    if (ch == '\n')
+	      *mapp++ = ' ';
+	    else {
+	      *mapp++ = '\\';
+	      *mapp++ = ch;
+	    }
+	  } else 
+	    *mapp++ = ch;
+	}
+	break;
+      case st_done:
+	/* Eat characters till there's no more output */
+	break;
       }
     }
+    quoted = 0;
+
+    /* Deal with stderr */
     if ( FD_ISSET(epipefd[0],&readfds) ) {
       if ( read(epipefd[0], &ch, 1) < 1 ) {
 	FD_CLR(epipefd[0], &ourfds);
@@ -174,7 +212,7 @@ int lookup_mount(const char *root, const char *name, int name_len,
 	*(errp++) = ch;
       }
     }
-  }
+}
 
   if ( mapp )
     *mapp = '\0';
@@ -190,16 +228,18 @@ int lookup_mount(const char *root, const char *name, int name_len,
     syslog(LOG_ERR, MODPREFIX "waitpid: %m");
     return 1;
   }
+
   if ( mapp == mapent || !WIFEXITED(status) || WEXITSTATUS(status) != 0 ) {
     syslog(LOG_NOTICE, MODPREFIX "lookup for %s failed", name);
     return 1;
   }
-  if ( (p = strchr(mapent,'\n')) ) *p = '\0';
 
   syslog(LOG_DEBUG, MODPREFIX "%s -> %s", name, mapent);
 
-  return ctxt->parse->parse_mount(root,name,name_len,mapent,
-				  ctxt->parse->context); 
+  ret = ctxt->parse->parse_mount(root, name, name_len,
+				 mapent,
+				 ctxt->parse->context); 
+  return ret;
 }
 
 int lookup_done(void *context)
