@@ -1,4 +1,4 @@
-#ident "$Id: lookup_yp.c,v 1.2 1997/10/06 21:52:03 hpa Exp $"
+#ident "$Id: lookup_yp.c,v 1.1.1.1 2003/09/09 11:22:10 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  lookup_yp.c - module for Linux automountd to access a YP (NIS)
@@ -14,9 +14,6 @@
  *
  * ----------------------------------------------------------------------- */
 
-/*
- */
-
 #include <stdio.h>
 #include <malloc.h>
 #include <errno.h>
