@@ -1,4 +1,4 @@
-#ident "$Id: mount_bind.c,v 1.2 2000/09/25 21:47:47 hpa Exp $"
+#ident "$Id: mount_bind.c,v 1.2 2003/09/10 14:27:41 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  mount_bind.c      - module to mount a local filesystem if possible;
@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <malloc.h>
+#include <alloca.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -31,8 +32,50 @@
 #define MODPREFIX "mount(bind): "
 int mount_version = AUTOFS_MOUNT_VERSION; /* Required by protocol */
 
+static int bind_works = 0;
+
 int mount_init(void **context)
 {
+  char *tmp1 = tempnam(NULL, "auto");
+  char *tmp2 = tempnam(NULL, "auto");
+  int err;
+  struct stat st1, st2;
+
+  if (tmp1 == NULL || tmp2 == NULL) {
+    if (tmp1)
+      free(tmp1);
+    if (tmp2)
+      free(tmp2);
+    return 0;
+  }
+
+  if (mkdir(tmp1, 0700) == -1)
+    goto out2;
+
+  if (mkdir(tmp2, 0700) == -1)
+    goto out1;
+
+  if (lstat(tmp1, &st1) == -1)
+    goto out;
+    
+  err = spawnl(LOG_DEBUG, PATH_MOUNT, PATH_MOUNT, "--bind", tmp1, tmp2, NULL);
+
+  if (err == 0 &&
+      lstat(tmp2, &st2) == 0 &&
+      st1.st_dev == st2.st_dev &&
+      st1.st_ino == st2.st_ino) {
+    bind_works = 1;
+  }
+  syslog(LOG_DEBUG, MODPREFIX "bind_works = %d\n", bind_works);
+  spawnl(LOG_DEBUG, PATH_UMOUNT, PATH_UMOUNT, tmp2, NULL);
+
+out:
+  rmdir(tmp2);
+out1:
+  free(tmp2);
+  rmdir(tmp1);
+out2:
+  free(tmp1);
   return 0;
 }
 
@@ -42,6 +85,7 @@ int mount_mount(const char *root, const char *name, int name_len,
 {
   char *fullpath;
   int err;
+  int i;
 
   fullpath = alloca(strlen(root)+name_len+2);
   if ( !fullpath ) {
@@ -49,36 +93,54 @@ int mount_mount(const char *root, const char *name, int name_len,
     return 1;
   }
   sprintf(fullpath, "%s/%s", root, name);
-    
-  syslog(LOG_DEBUG, MODPREFIX "calling mkdir %s", fullpath);
-  if ( mkdir(fullpath, 0555) && errno != EEXIST ) {
-    syslog(LOG_NOTICE, MODPREFIX "mkdir %s failed: %m", name);
-    return 1;
-  }
-    
-  syslog(LOG_DEBUG, MODPREFIX "calling mount --bind %s %s",
-	 fstype, what, fullpath);
-  err = spawnl(LOG_NOTICE, PATH_MOUNT, PATH_MOUNT, "--bind",
-	       what, fullpath, NULL);
-  
-  if ( err ) {
-    if ( rmdir(fullpath) && errno == EBUSY )
-      return 0;
+  i = strlen(fullpath);
+  while(--i > 0 && fullpath[i] == '/')
+    fullpath[i] = '\0';
 
-    syslog(LOG_DEBUG, MODPREFIX "failed to mount --bind %s on %s, trying symlink",
+  if (bind_works) {
+    syslog(LOG_DEBUG, MODPREFIX "calling mkdir_path %s", fullpath);
+    if ( mkdir_path(fullpath, 0555) && errno != EEXIST ) {
+      syslog(LOG_NOTICE, MODPREFIX "mkdir_path %s failed: %m", fullpath);
+      return 1;
+    }
+   
+    syslog(LOG_DEBUG, MODPREFIX "calling mount --bind %s %s",
 	   what, fullpath);
+    err = spawnl(LOG_NOTICE, PATH_MOUNT, PATH_MOUNT, "--bind",
+		 what, fullpath, NULL);
+
+    if ( err ) {
+      rmdir_path(fullpath);
+      return 1;
+    } else {
+      syslog(LOG_DEBUG, MODPREFIX "mounted %s type %s on %s",
+	     what, fstype, fullpath);
+      return 0;
+    }
+  } else {
+    char *cp;
+    char *basepath = alloca(strlen(fullpath) + 1);
+
+    strcpy(basepath, fullpath);
+    cp = strrchr(basepath, '/');
+    if (cp != NULL && cp != basepath)
+      *cp = '\0';
+
+    syslog(LOG_DEBUG, MODPREFIX "calling mkdir_path %s", basepath);
+    if ( mkdir_path(basepath, 0555) ) {
+      syslog(LOG_NOTICE, MODPREFIX "mkdir_path %s failed: %m", basepath);
+      return 1;
+    }
+    
     if ( symlink(what, fullpath) && errno != EEXIST ) {
       syslog(LOG_NOTICE, MODPREFIX "failed to create local mount %s -> %s", fullpath, what);
+      rmdir_path(fullpath);
       
       return 1;
     } else {
       syslog(LOG_DEBUG, MODPREFIX "symlinked %s -> %s", fullpath, what);
       return 0;
     }
-  } else {
-    syslog(LOG_DEBUG, MODPREFIX "mounted %s type %s on %s",
-	   what, fstype, fullpath);
-    return 0;
   }
 }
 
