@@ -1,10 +1,10 @@
-#ident "$Id: mount_generic.c,v 1.4 1998/04/01 11:15:47 hpa Exp $"
+#ident "$Id: mount_generic.c,v 1.2 2003/09/10 14:27:41 raven Exp $"
 /* ----------------------------------------------------------------------- *
  *   
  *  mount_generic.c - module for Linux automountd to mount filesystems
  *                    for which no special magic is required
  *
- *   Copyright 1997 Transmeta Corporation - All Rights Reserved
+ *   Copyright 1997-1999 Transmeta Corporation - All Rights Reserved
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 #include <stdio.h>
 #include <malloc.h>
+#include <alloca.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -50,9 +51,9 @@ int mount_mount(const char *root, const char *name, int name_len,
   }
   sprintf(fullpath, "%s/%s", root, name);
     
-  syslog(LOG_DEBUG, MODPREFIX "calling mkdir %s", fullpath);
-  if ( mkdir(fullpath, 0555) && errno != EEXIST ) {
-    syslog(LOG_NOTICE, MODPREFIX "mkdir %s failed: %m", name);
+  syslog(LOG_DEBUG, MODPREFIX "calling mkdir_path %s", fullpath);
+  if ( mkdir_path(fullpath, 0555) && errno != EEXIST ) {
+    syslog(LOG_NOTICE, MODPREFIX "mkdir_path %s failed: %m", name);
     return 1;
   }
     
@@ -68,7 +69,7 @@ int mount_mount(const char *root, const char *name, int name_len,
 		   what, fullpath, NULL);
   }
   if ( err ) {
-    rmdir(fullpath);
+    rmdir_path(fullpath);
     syslog(LOG_NOTICE, MODPREFIX "failed to mount %s (type %s) on %s",
 	   what, fstype, fullpath);
     return 1;
