@@ -1,11 +1,23 @@
-/*
- * parse_sun.c
+#ident "$Id: parse_sun.c,v 1.3 2003/09/10 14:27:41 raven Exp $"
+/* ----------------------------------------------------------------------- *
+ *   
+ *  parse_sun.c - module for Linux automountd to parse a Sun-format
+ *                automounter map
+ * 
+ *   Copyright 1997 Transmeta Corporation - All Rights Reserved
+ *   Copyright 2000 Jeremy Fitzhardinge <jeremy@goop.org>
  *
- * Module for Linux automountd to parse a Sun-format automounter map
- */
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA 02139,
+ *   USA; either version 2 of the License, or (at your option) any later
+ *   version; incorporated herein by reference.
+ *
+ * ----------------------------------------------------------------------- */
 
 #include <stdio.h>
 #include <malloc.h>
+#include <alloca.h>
 #include <errno.h>
 #include <netdb.h>
 #include <fcntl.h>
@@ -80,7 +92,8 @@ static void kill_context(struct parse_context *ctxt)
 }
 
 /* Find the $-variable matching a certain string fragment */
-static struct substvar *findvar(struct substvar *sv, char *str, int len)
+static const struct substvar *findvar(const struct substvar *sv,
+				      const char *str, int len)
 {
   while ( sv ) {
     if ( !strncmp(str,sv->def,len) && sv->def[len] == '\0' )
@@ -92,12 +105,13 @@ static struct substvar *findvar(struct substvar *sv, char *str, int len)
 
 /* $- and &-expand a Sun-style map entry and return the length of the entry.
    If "dst" is NULL, just count the length. */
-static int expandsunent(char *src, char *dst, char *key, struct substvar *svc,
-			int slashify_colons)
+static int expandsunent(const char *src, char *dst, const char *key,
+			const struct substvar *svc, int slashify_colons)
 {
-  struct substvar *sv;
+  const struct substvar *sv;
   int len, l, seen_colons;
-  char *p, ch;
+  const char *p;
+  char ch;
 
   len = 0;
   seen_colons = 0;
@@ -148,13 +162,15 @@ static int expandsunent(char *src, char *dst, char *key, struct substvar *svc,
       break;
 
     case '\\':
-      ch = *src;
-      if ( ch ) {
-	src++;
-	if ( dst ) *(dst++) = ch;
+      len++;
+      if (dst)
+	*dst++ = ch;
+
+      if (*src) {
 	len++;
-      } else {
-	; /* Terminal backslash, just eat it */
+	if (dst)
+	  *dst++ = *src;
+	src++;
       }
       break;
 
@@ -165,6 +181,9 @@ static int expandsunent(char *src, char *dst, char *key, struct substvar *svc,
       break;
 
     default:
+      if (isspace(ch))
+	seen_colons = 0;
+
       if ( dst ) *(dst++) = ch;
       len++;
       break;
@@ -176,7 +195,7 @@ static int expandsunent(char *src, char *dst, char *key, struct substvar *svc,
 
 /* Skip whitespace in a string; if we hit a #, consider the rest of the
    entry a comment */
-char *skipspace(char *whence)
+const char *skipspace(const char *whence)
 {
   while(1) {
     switch(*whence) {
@@ -189,9 +208,11 @@ char *skipspace(char *whence)
     case '\r':
       whence++;
       break;
-    case '\0':
-    case '#':
-      return NULL;		/* End of string */
+    case '#':			/* comment: skip to end of string */
+      while(*whence != '\0')
+	whence++;
+      /* FALLTHROUGH */
+
     default:
       return whence;
     }
@@ -199,11 +220,17 @@ char *skipspace(char *whence)
 }
 
 /* Get the length of a chunk delimitered by whitespace */
-int chunklen(char *whence)
+int chunklen(const char *whence)
 {
   int n = 0;
-  while(1) {
+  int quote = 0;
+
+  for(; *whence; whence++, n++) {
     switch(*whence) {
+    case '\\':
+      quote = 1;
+      continue;
+
     case ' ':
     case '\b':
     case '\t':
@@ -213,18 +240,21 @@ int chunklen(char *whence)
     case '\r':
     case '#':
     case '\0':
-      return n;
+      if (!quote)
+	return n;
+      /* FALLTHROUGH */
     default:
       break;
     }
-    n++;
-    whence++;
+    quote = 0;
   }
+
+  return n;
 }
 
 /* Compare str with pat.  Return 0 if compare equal or str is an abbreviation of
    pat of no less than mchr characters. */
-int strmcmp(char *str, char *pat, int mchr)
+int strmcmp(const char *str, const char *pat, int mchr)
 {
   int nchr = 0;
 
@@ -239,11 +269,12 @@ int strmcmp(char *str, char *pat, int mchr)
   return *pat - *str;
 }
 
-int parse_init(int argc, char **argv, void **context)
+int parse_init(int argc, const char * const *argv, void **context)
 {
   struct parse_context *ctxt;
   struct substvar *sv;
   char *noptstr;
+  const char *xopt;
   int optlen, len;
   int i, bval;
 
@@ -307,16 +338,15 @@ int parse_init(int argc, char **argv, void **context)
 
       case '-':
 	if ( !strncmp(argv[i]+2, "no-", 3) ) {
-	  noptstr = argv[i]+5;
+	  xopt = argv[i]+5;
 	  bval = 0;
 	} else {
-	  noptstr = argv[i]+2;
+	  xopt = argv[i]+2;
 	  bval = 1;
 	}
 
-	if ( strmcmp(noptstr, "slashify-colons", 1) ) {
+	if ( strmcmp(xopt, "slashify-colons", 1) )
 	  ctxt->slashify_colons = bval;
-	}
 	else
 	  syslog(LOG_ERR, MODPREFIX "unknown option: %s", argv[i]);
 
@@ -366,12 +396,175 @@ int parse_init(int argc, char **argv, void **context)
   }
 }
 
-int parse_mount(char *root, char *name, int name_len, char *mapent, void *context)
+static char *dequote(const char *str, int strlen)
+{
+  char *ret = malloc(strlen+1);
+  char *cp = ret;
+  const char *scp;
+  int origlen = strlen;
+
+  if (ret == NULL)
+    return NULL;
+
+  for(scp = str; strlen > 0 && *scp; scp++, strlen--) {
+    if (*scp == '\\')
+      continue;
+    *cp++ = *scp;
+  }
+  *cp = '\0';
+
+  syslog(LOG_DEBUG, MODPREFIX "dequote(\"%.*s\") -> %s", origlen, str, ret);
+
+  return ret;
+}
+
+static const char *parse_options(const char *str, char **ret)
+{
+  const char *cp = str;
+  int len;
+
+  if (*cp++ != '-')
+    return str;
+
+  if (*ret != NULL)
+    free(*ret);
+  
+  *ret = dequote(cp, len = chunklen(cp));
+
+  return cp + len;
+}
+
+static char *concat_options(char *left, char *right)
+{
+  char *ret;
+
+  if (left == NULL || *left == '\0') {
+    free(left);
+    ret = strdup(right);
+    return ret;
+  }
+
+  if (right == NULL || *right == '\0') {
+    free(right);
+    return strdup(left);
+  }
+
+  ret = malloc(strlen(left) + strlen(right) + 2);
+
+  if (ret == NULL) {
+    syslog(LOG_ERR, MODPREFIX "concat_options malloc: %m");
+    return NULL;
+  }
+
+  sprintf(ret, "%s,%s", left, right);
+
+  free(left);
+  free(right);
+
+  return ret;
+}
+
+static int sun_mount(const char *root, const char *name, int namelen,
+		     const char *path, int pathlen,
+		     const char *loc, int loclen,
+		     const char *options)
+{
+  char *fstype = "nfs";		/* Default filesystem type */
+  int nonstrict = 0;
+  int rv;
+  char *mountpoint;
+  char *what;
+
+  if (*options == '\0')
+    options = NULL;
+
+  if (options) {
+    char *noptions;
+    const char *comma;
+    char *np;
+    int len = strlen(options)+1;
+
+    noptions = np = alloca(len);
+    *np = '\0';
+
+    /* Extract fstype= pseudo option */
+    for(comma = options; *comma != '\0'; ) {
+      const char *cp;
+
+      while(*comma == ',')
+	comma++;
+
+      cp = comma;
+
+      while(*comma != '\0' && *comma != ',')
+	comma++;
+
+      if (strncmp("fstype=", cp, 7) == 0) {
+	int typelen = comma-(cp+7);
+	fstype = alloca(typelen+1);
+	memcpy(fstype, cp+7, typelen);
+	fstype[typelen] = '\0';
+      } else if (strncmp("nonstrict", cp, 9) == 0) {
+	nonstrict = 1;
+      } else {
+	memcpy(np, cp, comma-cp+1);
+	np += comma-cp+1;
+      }
+    }
+    if (np > noptions)
+      np[-1] = '\0';
+
+    options = noptions;
+  }
+
+  while (*path == '/') {
+    path++;
+    pathlen--;
+  }
+  mountpoint = alloca(namelen + pathlen + 2);
+  
+  if (pathlen)
+    sprintf(mountpoint, "%.*s/%.*s", namelen, name, pathlen, path);
+  else
+    sprintf(mountpoint, "%.*s", namelen, name);
+
+  what = alloca(loclen + 1);
+  memcpy(what, loc, loclen);
+  what[loclen] = '\0';
+
+  syslog(LOG_DEBUG, 
+	 MODPREFIX "mounting root %s, mountpoint %s, what %s, fstype %s, options %s\n",
+	 root, mountpoint, what, fstype, options);
+
+  if ( !strcmp(fstype, "nfs") ) {
+    rv = mount_nfs->mount_mount(root, mountpoint, strlen(mountpoint),
+				what, fstype, options,
+				mount_nfs->context);
+  } else {
+    /* Generic mount routine */
+    rv = do_mount(root, mountpoint, strlen(mountpoint), what, fstype, options);
+  }
+
+  if (nonstrict && rv) {
+    syslog(LOG_DEBUG, "ignoring failure of non-strict mount");
+    return 0;
+  }
+  return rv;
+}
+    
+/*
+ * syntax is:
+ *	[-options] location
+ *	[-options] [mountpoint [-options] location]...
+ */
+int parse_mount(const char *root, const char *name,
+		int name_len, const char *mapent, void *context)
 {
   struct parse_context *ctxt = (struct parse_context *) context;
-  char *pmapent, *options, *noptions, *ent, *p, *q, *fstype;
+  char *pmapent, *options;
+  const char *p;
   int mapent_len, rv;
-  int l, optlen;
+  int optlen;
 
   mapent_len = expandsunent(mapent,NULL,name,ctxt->subst,ctxt->slashify_colons);
   pmapent = alloca(mapent_len + 1);
@@ -381,7 +574,7 @@ int parse_mount(char *root, char *name, int name_len, char *mapent, void *contex
   }
   expandsunent(mapent,pmapent,name,ctxt->subst,ctxt->slashify_colons);
 
-  syslog(LOG_DEBUG, "expanded entry: %s", pmapent);
+  syslog(LOG_DEBUG, MODPREFIX "expanded entry: %s", pmapent);
 
   options = strdup(ctxt->optstr ? ctxt->optstr : "");
   if ( !options ) {
@@ -390,98 +583,123 @@ int parse_mount(char *root, char *name, int name_len, char *mapent, void *contex
   }
   optlen = strlen(options);
 
-  p = pmapent;
-  ent = NULL;
+  p = skipspace(pmapent);
 
-  while ( (p = skipspace(p)) ) {
-    if ( *p == '-' ) {
-      l = chunklen(++p);
-      if ( !l )
-	continue;
-      noptions = realloc(options, optlen+l+(optlen ? 2 : 1));
-      if ( !noptions ) {
-	free(options);
-	syslog(LOG_ERR, MODPREFIX "realloc: %m");
+  /* Deal with 0 or more options */
+  if (*p == '-') {
+    do {
+      char *noptions = NULL;
+
+      p = parse_options(p, &noptions);
+      options = concat_options(options, noptions);
+      
+      if (options == NULL) {
+	syslog(LOG_ERR, MODPREFIX "concat_options: %m");
 	return 1;
       }
-      if ( optlen ) noptions[optlen++] = ',';
-      memcpy(noptions+optlen, p, l);
-      noptions[optlen += l] = '\0';
-      options = noptions;
-      p += l;
-      syslog(LOG_DEBUG, MODPREFIX "gathered options: %s", options);
-    } else if ( *p == '/' ) {
-      /* Uh-oh, a multientry; don't support now, but ****FIXME**** */
-      /* We do tolerate "multientries" that only contain / though... */
-      if ( chunklen(p) == 1 ) {
-	p++;
-      } else {
-	free(options);
-	syslog(LOG_NOTICE, MODPREFIX "entry %s is a multipath entry", name);
-	return 1;
-      }
-    } else {
-      p[chunklen(p)] = '\0';	/* We don't use anything after the entry */
-      if ( *p == ':' ) p++;	/* Sun escape for entries starting with / */
-      if ( ! *p ) {
-	syslog(LOG_ERR, MODPREFIX "entry %s is empty!", name);
-	free(options);
-	return 1;
-      }
-      ent = p;
-      syslog(LOG_DEBUG, MODPREFIX "core of entry: %s", ent);
-      break;			/* We have the core entry, run w/it */
-    }
+      p = skipspace(p);
+    } while(*p == '-');
   }
+  syslog(LOG_DEBUG, MODPREFIX "gathered options: %s", options);
 
-  fstype = "nfs";		/* Default filesystem type */
+  if (*p == '/') {
+    int l;
 
-  /* Scan for -fstype= option, which isn't a real mount option */
+    /* It's a multi-mount; deal with it */
+    do {
+      char *myoptions = strdup(options);
+      char *path, *loc;
+      int pathlen, loclen;
+
+      if (myoptions == NULL) {
+	syslog(LOG_ERR, MODPREFIX "multi strdup: %m");
+	free(options);
+	return 1;
+      }
+
+      path = dequote(p, l = chunklen(p));
+      pathlen = strlen(path);
+
+      p += l;
+      p = skipspace(p);
+
+      /* Local options are appended to per-map options */
+      if (*p == '-') {
+	do {
+	  char *newopt = NULL;
+
+	  p = parse_options(p, &newopt);
+	  myoptions = concat_options(myoptions, newopt);
+
+	  if (myoptions == NULL) {
+	    syslog(LOG_ERR, MODPREFIX "multi concat_options: %m");
+	    free(options);
+	    free(path);
+	    return 1;
+	  }
+	  p = skipspace(p);
+	} while (*p == '-');
+      }
+
+      loc = dequote(p, l = chunklen(p));
+      loclen = strlen(loc);
+
+      if (loc == NULL || path == NULL) {
+	syslog(LOG_ERR, MODPREFIX "out of memory");
+	free(loc);
+	free(path);
+	free(options);
+	return 1;
+      }
+
+      p += l;
+      p = skipspace(p);
+
+      syslog(LOG_DEBUG, MODPREFIX "multimount: %.*s on %.*s with options %s",
+	     loclen, loc, pathlen, path, myoptions);
+
+      rv = sun_mount(root, name, name_len, path, pathlen, loc, loclen, myoptions);
+      free(path);
+      free(loc);
+      free(myoptions);
+
+      if (rv)
+	break;
+
+    } while(*p == '/');
 
-  noptions = strdup(options);
-  if ( !noptions ) {
-    syslog(LOG_ERR, MODPREFIX "strdup: %m");
     free(options);
-    return 1;
-  }
+    return rv;
+  } else {
+    /* Normal (non-multi) entries */
+    char *loc;
+    int loclen;
 
-  p = options;
+    if ( *p == ':' ) p++;	/* Sun escape for entries starting with / */
 
-  for ( q = strtok(noptions, ",") ; q ; q = strtok(NULL, ",") ) {
-    if ( !strncmp(q, "fstype=", 7) ) {
-      fstype = alloca(strlen(q+7)+1);
-      if ( !fstype ) {
-	syslog(LOG_ERR, MODPREFIX "alloca: %s");
-	return 1;
-      }
-      strcpy(fstype, q+7);
-    } else {
-      l = strlen(q);
-      memcpy(p, q, l);
-      p += l;
-      *p = ',';
-      p++;
+    loc = dequote(p, chunklen(p));
+    loclen = strlen(loc);
+
+    if (loc == NULL) {
+      syslog(LOG_ERR, MODPREFIX "out of memory");
+      free(loc);
+      free(options);
+      return 1;
     }
+    if (loclen == 0) {
+      syslog(LOG_ERR, MODPREFIX "entry %s is empty!", name);
+      free(options);
+      return 1;
+    }
+
+    syslog(LOG_DEBUG, MODPREFIX "core of entry: options=%s, loc=%.*s",
+	   options, loclen, loc);
+
+    rv = sun_mount(root, name, name_len, "/", 1, loc, loclen, options);
+    free(loc);
+    free(options);
   }
 
-  if ( p > options ) {
-    --p;			/* Delete last comma */
-    *p = '\0';
-  } else {
-    free(options);		/* No options - mount modules want NULL here */
-    options = NULL;
-  }
-  free(noptions);
-
-  if ( !strcmp(fstype, "nfs") ) {
-    rv = mount_nfs->mount_mount(root, name, name_len, ent, fstype, options,
-				mount_nfs->context);
-  } else {
-    /* Generic mount routine */
-    rv = do_mount(root, name, name_len, ent, fstype, options);
-  }
-
-  if (options) free(options);
   return rv;
 }
 
